/* Sniffit Data File                                                       */

#ifndef _SNIFFIT_H_
#define _SNIFFIT_H_

#include "pcap.h"

/**** Sniffit functions *****************************************************/
extern char *strlower (char *);
#endif
